/*
 * Created on Aug 15, 2003
 */
package org.oasis.was;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URL;
import java.util.HashMap;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.ValidationEvent;
import javax.xml.bind.ValidationEventHandler;
import javax.xml.bind.ValidationException;
import javax.xml.bind.Validator;

/**
 * @author Andrew Jaquith
 */
public class XMLSerializer implements ValidationEventHandler {

  private static final XMLSerializer INSTANCE = new XMLSerializer(); 
  public HashMap marshallers;
  public HashMap unmarshallers;
  private JAXBContext jaxbContext;
  private Object factory;
  private boolean isInited;
  private String noSchemaNamespace;
  

  private XMLSerializer() {
    isInited = false;
  }

  public static XMLSerializer initialize(Object factory, String jaxbPackage, String noSchemaNamespace) {
    INSTANCE.noSchemaNamespace = noSchemaNamespace;
    try {
      INSTANCE.jaxbContext = JAXBContext.newInstance(jaxbPackage);
      INSTANCE.factory = factory;
      INSTANCE.marshallers = new HashMap();
      INSTANCE.unmarshallers = new HashMap();
    }
    catch (JAXBException e) {
      System.out.println("Problem creating serializer: " 
        + e.getMessage());
    }
    INSTANCE.isInited = true;
    return INSTANCE;
  }
  
  public static XMLSerializer getInstance() {
    if (!INSTANCE.isInited) {
      throw new IllegalStateException("XMLSerializer.initialize() must be executed first.");
    }
    return INSTANCE;
  }

  public Object getObjectFactory() {
    return factory;
  }
  
  public void marshall(Object object, URL url, boolean validate) {
    Marshaller marshaller;
    System.out.println("Serializing '" + object + "' to URL '" + url + "' (validation=" + validate + ")");
    try {

      // Obtain a marshaller for the object; we will re-use one
      // for the given URL if it exists
      marshaller = (Marshaller)marshallers.get(object);
      if (marshaller == null) {
        marshaller = jaxbContext.createMarshaller();
        marshallers.put(object, marshaller);
      }
      marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, new Boolean(true));

      //We need the 'noNameSpace' property set so that the XML output
      //isn't infested with spurious xlmns:="" attributes 
      marshaller.setProperty("jaxb.noNamespaceSchemaLocation", noSchemaNamespace);
      marshaller.setEventHandler(this);
      OutputStream out = new FileOutputStream(url.getFile());
      if (validate) {
        if (isValid(object)) {
          marshaller.marshal(object, out);
        }
      }
      else {
        marshaller.marshal(object, out);
      }
      System.out.println("Serialization complete.");
    }
    catch (IOException e) {
      System.out.println("Can't get OutputStream: "
        + e.getMessage());
    }
    catch (JAXBException e) {
      System.out.println("Problem serializing item: "
        + e.getMessage());
    }
  }

  public Object unmarshall(URL url, boolean validate) {
    Unmarshaller unmarshaller;
    Object object = new Object();
    System.out.println("Deserializing from URL '" + url + "' (validation=" + validate + ")");
    try {
      // Obtain a marshaller for the object; we will re-use one
      // for the given URL if it exists
      unmarshaller = (Unmarshaller)marshallers.get(url);
      if (unmarshaller == null) {
        unmarshaller = jaxbContext.createUnmarshaller();
        unmarshaller.setValidating(validate);
        unmarshallers.put(url, unmarshaller);
      }
      unmarshaller.setEventHandler(this);
      unmarshaller.setValidating(validate);
      object = unmarshaller.unmarshal(url);
    }
    catch (JAXBException e) {
      System.out.println("Problem deserializing item: "
        + e.getMessage());
    }
    return object;
  }

  private boolean isValid(Object graph) throws JAXBException {
    try {
      Validator validator = jaxbContext.createValidator();
      return validator.validate(graph);
    }
    catch (ValidationException e){
      System.out.println("Problem validating item: " 
        + e.getMessage());
      return false;
    }
  }
  
	/* (non-Javadoc)
	 * @see javax.xml.bind.ValidationEventHandler#handleEvent(javax.xml.bind.ValidationEvent)
	 */
	public boolean handleEvent(ValidationEvent event) {
		System.out.println(event.getMessage());
		return true;
	}
  
}
