/*
 * Created on Aug 22, 2003
 */
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;

import javax.xml.bind.JAXBException;

import org.oasis.was.AttackCharacteristic;
import org.oasis.was.AttackSurface;
import org.oasis.was.AttackSurfaceType;
import org.oasis.was.Condition;
import org.oasis.was.ConditionType;
import org.oasis.was.Consequence;
import org.oasis.was.ConsequenceType;
import org.oasis.was.ObjectFactory;
import org.oasis.was.ProfileGroupType;
import org.oasis.was.ProfileSubgroupType;
import org.oasis.was.SerializableVulnerability;
import org.oasis.was.Target;
import org.oasis.was.TargetType;
import org.oasis.was.XMLSerializer;

/**
 * @author Andrew Jaquith
 */
public class SerializerTester {

  private XMLSerializer serializer;
  private ObjectFactory factory;
  public static final String JAXB_PACKAGE = "org.oasis.was";  
  public static final String SCHEMA_NAMESPACE = "http://www.oasis.org/was";

  public SerializerTester() {
    factory = new ObjectFactory();
    serializer = XMLSerializer.initialize(factory, JAXB_PACKAGE, SCHEMA_NAMESPACE);
  }

  public void testwrite(URL url) {
    try {
      // Create a sample vulnerability
      SerializableVulnerability vuln;
      vuln = factory.createSerializableVulnerability();
      vuln.setName("IIS Unicode overflow.");
      vuln.setDescription("This is an IIS buffer overflow.");
      
      // Create attack characteristics
      AttackCharacteristic characteristic = factory.createAttackCharacteristic();
      characteristic.setName("Identity impersonation");
      characteristic.setGroup(ProfileGroupType.ACCESS_CONTROL);
      characteristic.setSubgroup(ProfileSubgroupType.AUTHORIZATION);
      vuln.getAttackCharacteristics().add(characteristic);
      
      // Create attack surface
      AttackSurface surface = factory.createAttackSurface();
      surface.setType(AttackSurfaceType.COMPONENT_BOUNDARY);
      surface.setName("Authentication system");
      vuln.getAttackSurfaces().add(surface);
      
      // Create target
      Target target = factory.createTarget();
      target.setType(TargetType.APPLICATION_COMPONENT);
      target.setName("inet.dll");
      vuln.setTarget(target);
      
      // Create conditions
      Condition condition = factory.createCondition();
      condition.setType(ConditionType.PORT);
      condition.setName("139");
      vuln.getConditions().add(condition);
      
      // Create consequence
      Consequence consequence = factory.createConsequence();
      consequence.setType(ConsequenceType.PRIVILEGE_ELEVATION);
      consequence.setName("Root access");
      vuln.getConsequences().add(consequence);
      
      // Serialize everything
      boolean validate = false;
      serializer.marshall(vuln, url, validate);
      System.out.println("Graph successfully marshalled to " + url + ".");
    }
    catch (JAXBException e) {
       System.out.println("Problem creating new item: " 
        + e.getMessage());
    }
  }
  
  private void testread(URL url) {
    Object graph;
    graph = serializer.unmarshall(url, false);
    SerializableVulnerability vuln = (SerializableVulnerability)graph;
    System.out.println("Vulnerability:");
    System.out.println("..Name: " + vuln.getName());
    System.out.println("..Description: " + vuln.getDescription());
    System.out.println("..AttackCharacteristics: ");
    for (Iterator ch = vuln.getAttackCharacteristics().iterator(); ch.hasNext();) {
      AttackCharacteristic c = (AttackCharacteristic)ch.next();
      System.out.println("....AttackCharacteristic:");
      System.out.println("......Name: " + c.getName());
      System.out.println("......Group: " + c.getGroup());
      System.out.println("......Subgroup: " + c.getSubgroup());
    }
    System.out.println("..AttackSurfaces: ");
    for (Iterator ch = vuln.getAttackSurfaces().iterator(); ch.hasNext();) {
      AttackSurface c = (AttackSurface)ch.next();
      System.out.println("....AttackSurface:");
      System.out.println("......Name: " + c.getName());
      System.out.println("......Type: " + c.getType());
    }

    Target target = vuln.getTarget();
    System.out.println("..TestTarget:");
    System.out.println("....Name:" + target.getName());
    System.out.println("....Type:" + target.getType());

    System.out.println("..Conditions: ");
    for (Iterator ch = vuln.getConditions().iterator(); ch.hasNext();) {
      Condition c = (Condition)ch.next();
      System.out.println("....Condition:");
      System.out.println("......Name: " + c.getName());
      System.out.println("......Type: " + c.getType());
    }
    System.out.println("..Consequences: ");
    for (Iterator ch = vuln.getConsequences().iterator(); ch.hasNext();) {
      Consequence c = (Consequence)ch.next();
      System.out.println("....Consequence:");
      System.out.println("......Name: " + c.getName());
      System.out.println("......Type: " + c.getType());
    }
    System.out.println("Graph successfully unmarshalled from " + url + ".");
  }

  public static void main(String[] args) {
    SerializerTester tester = new SerializerTester();
    if (args.length == 0) {
      System.out.println("Usage: SerializerTester filename.xml");
      System.exit(0);
    }
    try {
      tester.testwrite(new URL("file://" + args[0]));
      tester.testread(new URL("file://" + args[0]));
    }
    catch (MalformedURLException e) {
      System.out.println("Malformed URL: 'file://" + args[0] + "'");
    }  
  }
}
